﻿using System;
using System.Collections.ObjectModel;

namespace ExportToExcel_11._1
{
    public class SampleDataGenerator
    {
        public static Boolean ConstantUnitPrice;

        private static String[] Products;
        private static String[] SellerNames;
        private static String[] Cities;

        private static Random random = new Random();

        public static ObservableCollection<Sale> GenerateSales(int numberOfSales)
        {
            SampleDataGenerator.Products = "Accessories;Bikes;Clothing;Components".Split(';');
            SampleDataGenerator.SellerNames = "John Smith;Alfredo Fetuchini;Howard Sprouse;Russell Shorter;Nicholas Carmona;Larry Lieb;Carl Costello;Benjamin Dupree;Bryan Culver;Monica Freitag;Lydia Burson;Harold Garvin;Walter Pang;David Haley;Glenn Landeros;Harry Tyler;Stanley Brooker;Antonio Charbonneau;Brandon Mckim;Claudia Kobayashi;Benjamin Meekins;Mark Slater;Kathe Pettel;Elisa Longbottom".Split(';');
            SampleDataGenerator.Cities = "Sofia;London;New York;Berlin;Seattle;Mellvile;Tokyo".Split(';');

            ObservableCollection<Sale> Sales = new ObservableCollection<Sale>();
            for (double i = 0; i < numberOfSales; i++)
            {
                Sale sale = new Sale();
                sale.Date = GetRandomDate();
                sale.Product = GerRandomProduct();
                sale.NumberOfUnits = GetRandomNumUnits();
                sale.Seller = GetRandomSeller();
                Sales.Add(sale);
            }
            return Sales;
        }

        #region Get Random Objects
        private static Seller GetRandomSeller()
        {
            return new Seller()
            {
                City = GetRandomCity(),
                Name = GetRandomSellerName()
            };
        }

        private static string GetRandomSellerName()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return SampleDataGenerator.SellerNames[a.Next(SampleDataGenerator.SellerNames.Length)];
        }

        private static string GetRandomCity()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return SampleDataGenerator.Cities[a.Next(SampleDataGenerator.Cities.Length)];
        }

        private static int GetRandomNumUnits()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return a.Next(1, 500);
        }

        private static Product GerRandomProduct()
        {
            return new Product()
            {
                Name = GetRandomProductName(),
                Price = GetRandomPrice()
            };
        }

        private static double GetRandomPrice()
        {
            if (!SampleDataGenerator.ConstantUnitPrice)
            {
                Random a = new Random(SampleDataGenerator.random.Next());
                return a.NextDouble() * 100;
            }
            else
            {
                return 50;
            }
        }

        private static string GetRandomProductName()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return SampleDataGenerator.Products[a.Next(SampleDataGenerator.Products.Length)];
        }

        private static DateTime GetRandomDate()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            int day = a.Next(1, 28);
            int month = a.Next(1, 12);
            int year = a.Next(DateTime.Today.Year - 5, DateTime.Today.Year+1);
            return new DateTime(year, month, day);
        }
        #endregion Get Random Objects
    }

    public class Sale
    {
        public Product Product { get; set; }
        public Seller Seller { get; set; }
        public DateTime Date { get; set; }
        public string City
        {
            get { return Seller.City; }
            set { Seller.City = value; }
        }

        public int NumberOfUnits { get; set; }
        public double AmountOfSale
        {
            get { return UnitPrice * NumberOfUnits; }
            set { Product.Price = value / NumberOfUnits; }
        }
        public double UnitPrice
        {
            get { return Product.Price; }
            set { Product.Price = value; }
        }
    }

    public class Product
    {
        public string Name { get; set; }
        public double Price { get; set; }
    }

    public class Seller
    {
        public string Name { get; set; }
        public string City { get; set; }
    }
}
