﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;
using Infragistics.Olap.Xmla;

namespace ExportToExcel_11._1
{
    public abstract class SampleFlatDataSourceBase : FlatDataSource
    {
        public bool Switched = false;
        public bool IsExpanded { get; set; }

        public SampleFlatDataSourceBase()
        {
            this.IsExpanded = true;

            this.Cube = XmlaDataSource.GenerateInitialCube("Sale");

            SelectItems();

            this.ItemsSource = SampleDataGenerator.GenerateSales(500);

            CubeMetadata cubeMetadata = new CubeMetadata();
            cubeMetadata.DisplayName = "Sales Data";
            cubeMetadata.DataTypeFullName = typeof(Sale).FullName;
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "AmountOfSale",
                DisplayName = "Amount of sale",
                DisplayFormat = "{0:C2}"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "NumberOfUnits",
                DisplayName = "Units sold"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "UnitPrice",
                DisplayName = "Unit Price",
                DisplayFormat = "{o:C2}"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
            {
                SourcePropertyName = "UnitPrice",
                DimensionType = DimensionType.Measure,
                DisplayName = "Final Price",
                Aggregator = new CustomMeasureAggregator()

            });


            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "Product",
                DisplayName = "Product"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "City",
                DisplayName = "City"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "Seller",
                DisplayName = "Seller"
            });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "Date",
                DisplayName = "Date"
            });
            

            this.CubesSettings.Add(cubeMetadata);

            HierarchyDescriptor<Sale> sellerHierarchy = new HierarchyDescriptor<Sale>(p => p.Seller);
            sellerHierarchy.AddLevel(p =>
                "All sellers",
                "All sellers"
                );
            sellerHierarchy.AddLevel(p =>
                p.Seller.Name,
               "Seller name"
                );
            this.AddHierarchyDescriptor(sellerHierarchy);

            HierarchyDescriptor<Sale> productHierarchy = new HierarchyDescriptor<Sale>(p => p.Product);
            productHierarchy.AddLevel(p =>
                "All products",
                "All products"
                );
            productHierarchy.AddLevel(p =>
                p.Product.Name,
                "Product name"
                );
            this.AddHierarchyDescriptor(productHierarchy);

            HierarchyDescriptor<Sale> locationHierarchy = new HierarchyDescriptor<Sale>(p => p.City);
            locationHierarchy.AddLevel(p =>
                "All locations",
                "All locations"
                );
            locationHierarchy.AddLevel(p =>
                p.City,
                "City"
                );
            this.AddHierarchyDescriptor(locationHierarchy);
        }

        public virtual void SelectItems()
        {

        }    
    }
}