﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using System.IO;
using Infragistics.Documents.Excel;
using Infragistics.Controls.Grids;
using Infragistics.Olap.Data;

namespace ExportToExcel_11._1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            
        }

        private void ExportToExcel(object sender, RoutedEventArgs e)
        {
            //if data is still not loaded give a warning.
            if (this.pivotGrid.GridLayout == null || this.pivotGrid.GridLayout.Rows == null ||
                this.pivotGrid.GridLayout.Columns == null)
            {
                MessageBox.Show("Loading");
                return;
            }

            this.SaveExport(this.CreateWorkbook());
        }

        private Workbook CreateWorkbook()
        {
            //Calculate the dimensions of the header rows and columns
            int TopHeaderHeight = 0;
            if (columnHeadersCheckBox.IsChecked.Value)
            {
                TopHeaderHeight = this.pivotGrid.GridLayout.PrivateRowsForColumnPanel.Count;
            }

            int LeftHeaderWidth = 0;
            if (rowHeadersCheckBox.IsChecked.Value)
            {
                LeftHeaderWidth = this.pivotGrid.GridLayout.PrivateColumnsForRowPanel.Count;
            }

            //Create the blank workbook and the sheet in it that will contain the PivotGrid data
            Workbook dataWorkbook = new Workbook();
            Worksheet sheetOne = dataWorkbook.Worksheets.Add("PivotWorkSheet");

            //Freeze header rows and columns
            if (columnHeadersCheckBox.IsChecked.Value || rowHeadersCheckBox.IsChecked.Value)
                sheetOne.DisplayOptions.PanesAreFrozen = true;
            if (rowHeadersCheckBox.IsChecked.Value)
                sheetOne.DisplayOptions.FrozenPaneSettings.FrozenRows = TopHeaderHeight;
            if (columnHeadersCheckBox.IsChecked.Value)
                sheetOne.DisplayOptions.FrozenPaneSettings.FrozenColumns = LeftHeaderWidth;

            //Set default width for the columns
            sheetOne.DefaultColumnWidth = 5000;


            int ColumnSpan, ColumnId, RowSpan, RowId;
            string CellValue;

            if (columnHeadersCheckBox.IsChecked.Value)
            {
                // Build Column Header
                foreach (PivotColumnHeaderCell cell in this.pivotGrid.GridLayout.ColumnHeaderCells)
                {
                    ColumnSpan = cell.RealColumnSpan;
                    ColumnId = this.pivotGrid.GridLayout.Columns.IndexOf(cell.Column as PivotDataColumn) +
                               LeftHeaderWidth;

                    RowSpan = cell.RowSpan;
                    RowId = this.pivotGrid.GridLayout.PrivateRowsForColumnPanel.IndexOf(cell.Row as PivotHeaderRow);

                    CellValue = cell.Member.Caption;
                    if (cell.Member.IsTotal && cell.IsToggleVisible == false)
                        CellValue += " " + "Excel Export Total";

                    if (ColumnSpan > 1 || RowSpan > 1)
                        sheetOne.MergedCellsRegions.Add(RowId, ColumnId, RowId + RowSpan - 1, ColumnId + ColumnSpan - 1);

                    this.SetCellValue(sheetOne.Rows[RowId].Cells[ColumnId], CellValue, "TopHeader");
                }
            }

            if (rowHeadersCheckBox.IsChecked.Value)
            {
                // Build Row Header
                foreach (PivotRowHeaderCell cell in this.pivotGrid.GridLayout.RowHeaderCells)
                {
                    ColumnSpan = cell.ColumnSpan;
                    ColumnId =
                        this.pivotGrid.GridLayout.PrivateColumnsForRowPanel.IndexOf(cell.Column as PivotHeaderColumn);

                    RowSpan = cell.RealRowSpan;
                    RowId = this.pivotGrid.GridLayout.Rows.IndexOf(cell.Row as PivotDataRow) + TopHeaderHeight;

                    CellValue = cell.Member.Caption;
                    if (cell.Member.IsTotal && cell.IsToggleVisible == false)
                        CellValue += " " + "Excel Export Total" ;

                    if (ColumnSpan > 1 || RowSpan > 1)
                        sheetOne.MergedCellsRegions.Add(RowId, ColumnId, RowId + RowSpan - 1, ColumnId + ColumnSpan - 1);

                    this.SetCellValue(sheetOne.Rows[RowId].Cells[ColumnId], CellValue, "LeftHeader");
                }
            }

            // Build Data
            int currentRow = 0;
            foreach (PivotDataRow row in this.pivotGrid.GridLayout.Rows)
            {
                int currentCell = 0;
                foreach (PivotCell cell in row.Cells)
                {
                    string currentCellValue;
                    if (cell.Data != null)
                    {
                        currentCellValue = (cell.Data as ICell).FormattedValue;
                    }
                    else
                    {
                        currentCellValue = string.Empty;
                    }
                    this.SetCellValue(sheetOne.Rows[currentRow + TopHeaderHeight].Cells[currentCell + LeftHeaderWidth],
                                      currentCellValue, "Data");
                    currentCell++;
                }
                currentRow++;
            }

            //return the filled in workbook
            return dataWorkbook;
        }


        //Set the value and format each cell
        private void SetCellValue(WorksheetCell cell, string CellValue, string CellType)
        {
            cell.Value = CellValue;
            cell.CellFormat.ShrinkToFit = ExcelDefaultableBoolean.True;
            cell.CellFormat.VerticalAlignment = VerticalCellAlignment.Center;
            cell.CellFormat.Alignment = HorizontalCellAlignment.Center;

            if (CellType == "TopHeader")
            {
                cell.CellFormat.FillPatternForegroundColor = System.Drawing.Color.FromArgb(255, 181, 212, 240);
                cell.CellFormat.FillPattern = FillPatternStyle.Solid;
            }
            else if (CellType == "LeftHeader")
            {
                cell.CellFormat.FillPatternForegroundColor = System.Drawing.Color.FromArgb(255, 200, 212, 240);
                cell.CellFormat.FillPattern = FillPatternStyle.Solid;
            }
            else if (CellType == "Data")
            {
                cell.CellFormat.FillPatternForegroundColor = System.Drawing.Color.FromArgb(255, 242, 244, 249);
                cell.CellFormat.FillPattern = FillPatternStyle.Solid;
            }
            else
            {
                MessageBox.Show("Cell Type Not Located");
            }
        }

        //Open a dialog and prompt the user to save the Excel workbook with the exported data
        private void SaveExport(Workbook dataWorkbook)
        {
            SaveFileDialog dialog = new SaveFileDialog { Filter = "Excel files|*.xls", DefaultExt = "xls" };

            bool? showDialog = dialog.ShowDialog();
            if (showDialog == true)
            {
                using (Stream exportStream = dialog.OpenFile())
                {
                    dataWorkbook.Save(exportStream);
                    exportStream.Close();
                }
            }
        }
    }
}